#    Copyright (C) 1989, 1992 Aladdin Enterprises.  All rights reserved.
#    Distributed by Free Software Foundation, Inc.
#
# This file is part of Ghostscript.
#
# Ghostscript is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
# to anyone for the consequences of using it or for whether it serves any
# particular purpose or works at all, unless he says so in writing.  Refer
# to the Ghostscript General Public License for full details.
#
# Everyone is granted permission to copy, modify and redistribute
# Ghostscript, but only under the conditions described in the Ghostscript
# General Public License.  A copy of this license is supposed to have been
# given to you along with Ghostscript so you can know your rights and
# responsibilities.  It should be in a file named COPYING.  Among other
# things, the copyright notice and this notice must be preserved on all
# copies.

# makefile for Ghostscript, MS-DOS 5.0/DJ's GCC configuration.
# Note: this makefile assumes you are using gcc in ANSI mode.

# ------------------------------- Options ------------------------------- #

####### The following are the only parts of the file you should need to edit.

# ------ Generic options ------ #

# Define the default directory/ies for the runtime
# initialization and font files.  Separate multiple directories with ;.
# Use / to indicate directories, not a single \.

GS=gs
gsdatadir=c:/lib/gs
GS_LIB_DEFAULT=$(gsdatadir);$(gsdatadir)/kanji;$(gsdatadir)/fonts

# Define the name of the Ghostscript initialization file.
# (There is no reason to change this.)

GS_INIT=gs_init.ps

# Choose generic configuration options.

# -DDEBUG
#	includes debugging features (-Z switch) in the code.
#	  Code runs substantially slower even if no debugging switches
#	  are set.
# -DNOPRIVATE
#	makes private (static) procedures and variables public,
#	  so they are visible to the debugger and profiler.
#	  No execution time or space penalty.

GENOPT=

# ------ Platform-specific options ------ #

# Define the name of the C compiler.

CC=gcc


# a.out -> *.exe
#   If the version of your DJ's gpp is 1.12 or later,
# define this macro as 'coff2exe'.

AOUT2EXE=aout2exe

# XCFLAGS can be set from the command line.
# We don't include -ansi, because this gets in the way of the platform-
#   specific stuff that <math.h> typically needs; nevertheless, we expect
#   gcc to accept ANSI-style function prototypes and function definitions.

CFLAGS=-g -O $(XCFLAGS) -DCOMPFONT -DWMODE -DKANJI

# Define platform flags for ld.
# XLDFLAGS can be set from the command line.

LDFLAGS=$(XLDFLAGS)

# Define any extra libraries to link into the executable.
# (Libraries required by individual drivers are handled automatically.)

EXTRALIBS=

# ------ Devices and features ------ #

# Choose the machine dependent feature(s) to include. See djgpp.sj
# for details.
DJGPP_PLATFORMS=pc98_ pcat_

# Choose the language feature(s) to include.  See gs.mak for details.
FEATURE_DEVS=filter.dev dps.dev level2.dev \
	kanji.dev kfpc98.dev kfzeit.dev kfztbez.dev

# Choose the device(s) to include.  See devs.mak for details.
DEVICE_DEVS=
DEVICE_DEVS1=
DEVICE_DEVS2=
DEVICE_DEVS3=
DEVICE_DEVS4=
DEVICE_DEVS5=
DEVICE_DEVS6=
DEVICE_DEVS7=
DEVICE_DEVS8=
DEVICE_DEVS9=

# ---------------------------- End of options --------------------------- #

# Define the name of the makefile -- used in dependencies.

MAKEFILE=djgpp.mak

# Define the ANSI-to-K&R dependency.  (gcc accepts ANSI syntax.)

AK=

# Define the compilation rules and flags.

CCFLAGS=$(GENOPT) $(CFLAGS)

CCC=$(CC) $(CCFLAGS) -c
CCCF=$(CCC)
CCD=$(CCC)
CCINT=$(CCC)

# ----------------------------- Generic stuff ----------------------------- #

# Define the platform name.

PLATFORM=djgpp_

# Define the extensions for the object and executable files.

OBJ=o
XE=.exe

# Define the current directory prefix, shell quote string, and shell names.

EXP=
QQ=\"
SHELL=
SH=
SHP=

# Define the compilation rules and flags.

.c.o:
	$(CCC) $<

CC0=$(CCC)
CCINT=$(CCC)

# -------------------------- Include Makefiles ---------------------------- #

!include "gs.mak"		# Borland MAKE specific
!include "devs.mak"		# Borland MAKE specific

# -------------------------- Auxiliary programs --------------------------- #

ansi2knr$(XE): ansi2knr.c
	$(CC) $(CFLAGS) -o ansi2knr ansi2knr.c
	strip ansi2knr
	$(AOUT2EXE) ansi2knr

genarch$(XE): genarch.c
	$(CC) $(CFLAGS) -o genarch genarch.c
	strip genarch
	$(AOUT2EXE) genarch

genconf$(XE): genconf.c
	$(CC) $(CFLAGS) -o genconf genconf.c
	strip genconf
	$(AOUT2EXE) genconf

echogs$(XE): echogs.c
	$(CC) -o echogs $(CFLAGS) echogs.c
	strip echogs
	$(AOUT2EXE) echogs

# -------------------------------- Library -------------------------------- #

# The DJ's GCC platform

djgpp__=gp_djgpp.$(OBJ) gp_msdos.$(OBJ)
djgpp_.dev: $(djgpp__) $(DJGPP_PLATFORMS)
	$(SHP)gssetmod djgpp_ $(djgpp__)
	$(SHP)gsaddmod djgpp_ -include $(DJGPP_PLATFORMS)
	$(SHP)gsaddmod djgpp_ -ps gp_djgpp
	$(SHP)gsaddmod djgpp_ -oper gp_djgpp
	$(SHP)gsaddmod djgpp_ -lib pc

gp_djgpp.$(OBJ): gp_djgpp.c $(string__h) $(gx_h) $(gp_h) gp_djgpp.h gp_djg_.h
	$(CC) $(CFLAGS) -Wall -c gp_djgpp.c

gp_djg_.h: $(DJGPP_PLATFORMS)
	rm -f gp_djg_.h
	for %f in ($(DJGPP_PLATFORMS) nul) do type %f >> gp_djg_.h

pc98_: gp_pc98.$(OBJ)
	gssetdjg pc98 PC-9801 gp_pc98.$(OBJ)
gp_pc98.$(OBJ): gp_pc98.c djgpp.h gp_djgpp.h

pcat_: gp_pcat.$(OBJ)
	gssetdjg pcat PC-AT gp_pcat.$(OBJ)
gp_pcat.$(OBJ): gp_pcat.c djgpp.h gp_djgpp.h

# The platform-dependent drivers.

### --------------------- Kanji-font drivers --------------------- ###

### ---------------------- PC-9801 ROM font ---------------------- ###

zkfpc98.$(OBJ): zkfpc98.c $(OP) $(gsmatrix_h) $(state_h) $(store_h) $(alloc_h)

kfpc98_=zkfpc98.$(OBJ) 
kfpc98.dev: $(kfpc98_)
	$(SHP)gssetmod kfpc98 $(kfpc98_)
	$(SHP)gsaddmod kfpc98 -oper zkfpc98
	$(SHP)gsaddmod kfpc98 -ps kfpc98
	$(SHP)gsaddmod kfpc98 -lib pc

# ----------------------------- Main program ------------------------------ #

# Get around the fact that the DOS shell has a rather small limit on
# the length of a command line.  (sigh)

# Interpreter main program

GSDJGPP=gs.$(OBJ) gsmain.$(OBJ) $(INT) $(LIBGS) $(ALL_DEVS)

$(GS)$(XE): $(GSDJGPP) gs_djgpp.tr ld.tr
	$(CC) -o gs $(LDFLAGS) @gs_djgpp.tr @ld.tr $(EXTRALIBS) -lm
	strip gs
	$(AOUT2EXE) gs
